function registerVehiclePart(%vehicle, %type, %name, %count)
{
  if (!isObject(%vehicle))
  {
    return 0;
  }

  %vehicle.partCount[%type, %name] = %count;

  if (!%vehicle.partListHas[%type, %name])
  {
    %vehicle.partListHas[%type, %name] = 1;
    %vehicle.partIndex[%type, %name] = getFieldCount(%vehicle.partList[%type]);

    if (%vehicle.partList[%type] $= "")
    {
      %vehicle.partList[%type] = %name;
    }
    else
    {
      %vehicle.partList[%type] = %vehicle.partList[%type] TAB %name;
    }

  }

  if (!%vehicle.typeListHas[%type])
  {
    %vehicle.typeListHas[%type] = 1;

    if (%vehicle.typeList $= "")
    {
      %vehicle.typeList = %type;
    }
    else
    {
      %vehicle.typeList = %vehicle.typeList TAB %type;
    }
  }
}

function Vehicle::setPart(%this, %type, %name, %color)
{
  %vehicle = %this.getDataBlock();

  if (%type $= "Tire")
  {
    %tire = %vehicle.partCount[%type, %name];

    if (!isObject(%tire))
    {
      return;
    }

    %count = %this.getWheelCount();

    for (%i = 0; %i < %count; %i++)
    {
      %this.setWheelTire(%i, %tire);
    }

    %this.part["Tire"] = %name;
    return;
  }

  if (%vehicle.partCount[%type, %name] <= 0)
  {
    return;
  }

  %current = %this.part[%type];

  for (%i = 0; %i < %vehicle.partCount[%current]; %i++)
  {
    %this.hideNode(getPartNodeName(%type, %vehicle.partIndex[%type, %current], %i));
  }

  %this.part[%type] = %name;
  %this.partColor[%type] = %color;

  for (%i = 0; %i < %vehicle.partCount[%type, %name]; %i++)
  {
    %node = getPartNodeName(%type, %vehicle.partIndex[%type, %name], %i);

    %this.unHideNode(%node);
    %this.setNodeColor(%node, %color);
  }
}

function Vehicle::setDefaultParts(%this, %color)
{
  %this.hideAllParts();

  %vehicle = %this.getDataBlock();
  %count = getFieldCount(%vehicle.typeList);

  if (%color $= "")
  {
    %color = getWords(getColorIDTable(%this.spawnBrick.colorID), 0, 2) SPC 1;
  }

  for (%i = 0; %i < %count; %i++)
  {
    %type = getField(%vehicle.typeList, %i);
    %part = getField(%vehicle.partList[%type], 0);

    %this.setPart(%type, %part, %color);
  }
}

function Vehicle::setRandomParts(%this, %color)
{
  %this.hideAllParts();

  %vehicle = %this.getDataBlock();
  %count = getFieldCount(%vehicle.typeList);

  if (%color $= "")
  {
    %color = getWords(getColorIDTable(%this.spawnBrick.colorID), 0, 2) SPC 1;
  }

  for (%i = 0; %i < %count; %i++)
  {
    %type = getField(%vehicle.typeList, %i);

    %rand = getRandom(getFieldCount(%vehicle.partList[%type]) - 1);
    %part = getField(%vehicle.partList[%type], %rand);

    %this.setPart(%type, %part, %color);
  }
}

function Vehicle::hideAllParts(%this)
{
  %vehicle = %this.getDataBlock();
  %types = getFieldCount(%vehicle.typeList);

  for (%i = 0; %i < %types; %i++)
  {
    %type = getField(%vehicle.typeList, %i);

    if (%type $= "Tire")
    {
      continue;
    }

    %parts = getFieldCount(%vehicle.partList[%type]);

    for (%j = 0; %j < %parts; %j++)
    {
      %part = getField(%vehicle.partList[%type], %j);

      for (%k = 0; %k < %vehicle.partCount[%type, %part]; %k++)
      {
        %node = getPartNodeName(%type, %j, %k);
        %this.hideNode(%node);
      }
    }
  }
}

function getPartNodeName(%type, %i, %j)
{
  %alpha = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  return %type @ "_" @ getSubStr(%alpha, %i, 1) @ "_" @ getSubStr(%alpha, %j, 1);
}

function Vehicle::GarageDefault(%this, %a)
{
  //%this.setDefaultParts();
  %this.setRandomParts(0.3 + getRandom() * 4 SPC 0.3 + getRandom() * 4 SPC 0.3 + getRandom() * 4 SPC 1);
}

function Garage_AddParts(%vehicle, %fileName)
{
  %fp = new FileObject();
  %fp.openForRead(%fileName);

  while (!%fp.isEOF())
  {
    %line = %fp.readLine();
    %tag = getField(%line, 0);

    if (%tag $= "VEHICLE")
    {
      if (%gotVeh) break;
      %gotVeh = 1;
      continue;
    }

    registerVehiclePart(%vehicle, %tag, getField(%line, 1), getField(%line, 2));
  }
}